package algorithmrepository.differentialgeometry;
import static java.lang.Math.*;

/**
 * 
 * Coordinate order is R, phi, Z
 *
 */
public class CylinderGeometry extends GeometryImpl {

    @Override
    public double[][] metricTensor(double[] atCoords, int metricCoOrContra) {
        
        double R = atCoords[0];
        
        double[][] g = new double[3][3];
        g[0][0] = 1.0;
        g[1][1] = metricCoOrContra == COVARIANT ? R*R : 1.0/(R*R);
        g[2][2] = 1.0;
        
        return g;
    }
    
    @Override
    public double metricCoefficient(double[] atCoords, int i, int j, int coOrContra) {
        double R = atCoords[0];
        return i == j ? (i == 1 ? (coOrContra == COVARIANT ? R*R : 1.0/(R*R)) : 1.0) : 0.0;
    }

    @Override
    public double[][] cartesianBasisVectors(double[] pos, int coOrContra) {
        double R = pos[0], phi = pos[1], Z = pos[2];
        
        double[][] coBasis = new double[][] { 
            {cos(phi), sin(phi), 0},
            {-R*sin(phi), R*cos(phi), 0}, 
            {0, 0, 1} }; 
        
        if (coOrContra == COVARIANT) {
            return coBasis;
        } else {
            return flipCartesianBasis(pos, CONTRAVARIANT, coBasis);            
        }
    }

    @Override
    public double[] toCartesianCoords(double[] coords) {
        double R = coords[0], phi = coords[1], Z = coords[2];
        return new double[] { R*cos(phi), R*sin(phi), Z };
    }

    @Override
    public double[] fromCartesianCoords(double[] coordsXYZ) {
        double x = coordsXYZ[0], y = coordsXYZ[1], z = coordsXYZ[2];
        double R = sqrt(x*x + y*y);
        double phi = atan2(y, x);        
        return new double[] { R, phi, z  };
    }


    @Override
    public int dim() {
        return 3;
    }
    
  


}
