package algorithmrepository.differentialgeometry;

import algorithmrepository.Algorithms;

public class Play {

    public static void main(String[] args) throws Exception {
       //run1();
       //run2();
        run3();
       
    }
    
    public static void run3() throws Exception {
        Geometry g = new CartesianGeometry();
        double[] pos = new double[] { 1.0, 2.0, 3.0 };
        dump(g.metricTensor(pos, Geometry.COVARIANT));
        System.out.println();
        dump(g.metricTensorFromCoordinateTransform(pos, Geometry.COVARIANT));
        
        System.out.println();
        g = new CylinderGeometry();
        pos = new double[] { 2.0, 0.34, 3.0 };
        dump(g.metricTensor(pos, Geometry.COVARIANT));
        System.out.println();
        dump(g.metricTensorFromCoordinateTransform(pos, Geometry.COVARIANT));
        
       
       
           
    }
    
    public static void run2() throws Exception {
        
        Geometry g = new CartesianGeometry();
        double[][] pos = new double[][] { { 0.0 }, { 0.0 }, { 0.0 } };
        double[] du = { 2,2,2 };
        dump(g.dV(pos, du));
        
        g = new CylinderGeometry();
        pos = new double[][] { { 2.0 }, { 10.0 }, { 0.0 } };
        du = new double[] { 2,2,2 };
        dump(g.dV(pos, du));
        
        double[] R = linSpace(2.0, 2.0, 1000);
        double[] phi = linSpace(0,2*Math.PI, R.length);        
        double[] Z = linSpace(1.0, 1.0, R.length);

        double[] dl = g.dl(new double[][] { R, phi, Z }, 1, phi[1]-phi[0]);      
        System.out.println(Algorithms.sum(dl));
        
        System.out.println(2*Math.PI*2.0);
        
        double[] atCoords = new double[] { 1.2, 30.0*Math.PI/180.0,  -1.1 };
        dump(g.metricTensor(atCoords, Geometry.COVARIANT));
        dump(g.metricTensorFromCoordinateTransform(atCoords, Geometry.COVARIANT));
        
        System.out.println(g.metricCoefficientFromCoordinateTransform(atCoords, 1, 1, Geometry.COVARIANT));
        System.out.println(g.metricCoefficientFromCoordinateTransform(atCoords, 0, 1, Geometry.COVARIANT));
        System.out.println(g.metricCoefficientFromCoordinateTransform(atCoords, 0, 0, Geometry.COVARIANT));
        System.out.println(g.metricCoefficientFromCoordinateTransform(atCoords, 2, 2, Geometry.COVARIANT));
        
        System.out.println();
        
        System.out.println(g.metricCoefficientFromCoordinateTransform(atCoords, 1, 1, Geometry.CONTRAVARIANT));
        System.out.println(g.metricCoefficientFromCoordinateTransform(atCoords, 0, 1, Geometry.CONTRAVARIANT));
        System.out.println(g.metricCoefficientFromCoordinateTransform(atCoords, 0, 0, Geometry.CONTRAVARIANT));
        System.out.println(g.metricCoefficientFromCoordinateTransform(atCoords, 2, 2, Geometry.CONTRAVARIANT));

        System.out.println();        
        dump(g.metricTensor(atCoords, Geometry.CONTRAVARIANT));
        
        dump(g.metricTensorFromCoordinateTransform(atCoords, Geometry.CONTRAVARIANT));
    }
    
    
    public static void run1() throws Exception { 
        GeometryImpl g = new CartesianGeometry();
        
        double[] pos = new double[] { 0.0, 0.0, 0.0 };
        double[][] baseCo = g.cartesianBasisVectors(pos, Geometry.COVARIANT);
        dump(baseCo);
        System.out.println();
        dump(g.flipCartesianBasis(pos, Geometry.COVARIANT, baseCo));
        System.out.println();
        
        g = new CylinderGeometry();
        pos = new double[] { 2.0, 45.0*Math.PI/180.0, 0.5 };
        baseCo = g.cartesianBasisVectors(pos, Geometry.COVARIANT);
        dump(baseCo);        
        System.out.println();
        dump(g.flipCartesianBasis(pos, Geometry.COVARIANT, baseCo));               
    }
    
    
    public static void dump(double[][] a) {
        for(int i=0;i<a.length;++i) {
            dump(a[i]);
        }
    }
    
    public static void dump(double[] a) {
        System.out.print(a[0]);
        for(int i=1;i<a.length;++i) {
            System.out.print("\t"+a[i]);
        }
        System.out.println();
    }
    
    public final static double[] linSpace(double x0, double x1, int n){
        if(n == 1) return new double[]{ (x0+x1)/2 };
        double f[] = new double[n];
        double dx = (x1 - x0)/(double)(n - 1.0);
        for(int i=0; i<n; i++)
            f[i] = x0 + i*dx;
        return f;
    }

}
